from typing import List

from aldryn_search.helpers import get_plugin_index_data
from aldryn_search.utils import clean_join
from cms.models import CMSPlugin
from cms.models import Placeholder
from django.conf import settings
from django.http import HttpRequest

from djangocms_algolia.index import FakeCMSRequestFactor


def render_text_from_placeholders(placeholders: List[Placeholder], lang_code: str = settings.LANGUAGE_CODE) -> str:
    indexable_strings: List[str] = []
    for placeholder in placeholders:
        indexable_strings.append(render_text_from_placeholder(placeholder, lang_code))
    return clean_join(' ', indexable_strings)


def render_text_from_placeholder(placeholder: Placeholder, lang_code: str = settings.LANGUAGE_CODE) -> str:
    indexable_strings: List[str] = []
    plugins = CMSPlugin.objects.filter(language=lang_code).filter(placeholder=placeholder)
    for plugin in plugins:
        request_fake = FakeCMSRequestFactor().get_request()
        plugin_text_content = render_text_from_plugin(plugin, request_fake)
        indexable_strings.append(plugin_text_content)
    return clean_join(' ', indexable_strings)


def render_text_from_plugin(plugin: CMSPlugin, request: HttpRequest) -> str:
    return clean_join(' ', get_plugin_index_data(plugin, request))



#!/usr/bin/env python3
from setuptools import setup


from djangocms_algolia import __version__


setup(
    long_description_content_type='text/markdown',
    name='djangocms-algolia',
    version=__version__,
    author='Victor Yunenko',
    author_email='victor@what.digital',
    long_description=open('README.md').read(),
    
    url='https://gitlab.com/victor.yunenko/djangocms-algolia',
    packages=[
        'djangocms_algolia',
    ],
    include_package_data=True,
    install_requires=[
        'django >= 2.2, < 3',
        'django-cms >= 3.7.2, < 4',
        'algoliasearch-django',
        'aldryn-search',
        'beautifulsoup4',
        'django-standard-form == 1.1.1', # because the latest version 1.1.4 from divio isn't present on pypi.org
    ],
)



import re
from datetime import datetime
from typing import Union

from aldryn_search.search_indexes import TitleIndex
from algoliasearch_django import AlgoliaIndex
from algoliasearch_django.decorators import register
from cms.models import Title
from cms.test_utils.testcases import BaseCMSTestCase
from cms.toolbar.toolbar import CMSToolbar
from django.conf import settings
from django.db import connection
from django.db.models import QuerySet
from django.forms import Media
from django.http import HttpRequest
from django.test import Client
from django.utils import translation
from haystack.indexes import SearchIndex


def _is_proxy_model_creation_possible() -> bool:
    """
    A proxy model cannot be imported in django when the parent table doesn't exists.
    But algolia django package tries to import the models before the user runs `python manage.py migrate`,
    which throws a django exception.
    
    In other words algoliasearch-django package prevents you from creating a database in the first place.
    """
    return 'cms_title' in connection.introspection.table_names()


class FakeCMSRequestFactor(BaseCMSTestCase):
    client = Client
    
    def get_request(self, *args, **kwargs) -> HttpRequest:
        request = super().get_request(*args, **kwargs)
        request.placeholder_media = Media()
        request.session = {}
        request.toolbar = CMSToolbar(request)
        return request


if _is_proxy_model_creation_possible():

    class AlgoliaPageDataProxy(Title):
        model_type = 'page'

        def search_index_description(self) -> str:
            aldryn_haystack_index: Union[SearchIndex, TitleIndex] = TitleIndex()
            page_content: str = aldryn_haystack_index.get_search_data(
                obj=self,
                language=translation.get_language(),
                request=FakeCMSRequestFactor().get_request(),
            )
            if settings.ALGOLIA_SEARCH_INDEX_TEXT_LIMIT:
                return page_content[:settings.ALGOLIA_SEARCH_INDEX_TEXT_LIMIT]
            else:
                return page_content

        def pub_date(self) -> datetime:
            return self.page.publication_date

        def url(self) -> str:
            lang_prefix = f'/{translation.get_language()}/'
            url_with_lang = self.page.get_absolute_url()
            url_without_lang = re.sub(rf'^{lang_prefix}', '', url_with_lang)
            return url_without_lang

        class Meta:
            app_label = 'cms'
            proxy = True


    @register(AlgoliaPageDataProxy)
    class PageIndex(AlgoliaIndex):
        if hasattr(settings, 'DJANGO_ENV'):
            index_name =  f'{settings.DJANGO_ENV.value}_cms_pages'
        else:
            index_name =  'cms_pages'

        fields = [
            'title',
            'url',
            'pub_date',
            'meta_description',
            'search_index_description',
        ]

        # def __init__(self, model, client, settings):
        #     lang_code_current: str = translation.get_language()
        #     settings_with_postfix = settings or {}
        #     settings_with_postfix['INDEX_SUFFIX'] = lang_code_current
        #     super().__init__(model, client, settings)

        def get_queryset(self) -> QuerySet:
            aldryn_haystack_index: SearchIndex = TitleIndex()
            return aldryn_haystack_index.get_index_queryset(
                language=translation.get_language(),
            )



from typing import List

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from cms.models import Page


@apphook_pool.register
class DjangocmsAlgoliaApphook(CMSApp):
    app_name = 'djangocms_algolia'
    name = "Search"

    def get_urls(self, page: Page = None, language: str = None, **kwargs) -> List[str]:
        return [f'{self.app_name}.urls']
